<?php
/* 
 ======================================================================
 lastRSS 0.2
 
 Simple yet powerfull PHP class to parse RSS files.
 
 by Vojtech Semecky, webmaster@webdot.cz
 
 Latest version, features, more information and examples:
 	http://www.webdot.cz/lastrss/
 
 ----------------------------------------------------------------------
 LICENSE

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License (GPL)
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 To read the license please visit http://www.gnu.org/copyleft/gpl.html
 ======================================================================
*/

class lastRSS {

	// -------------------------------------------------------------------
	// Parse RSS file and returns associative array.
	// -------------------------------------------------------------------
	function Get ($rss_url) {
		// If CACHE ENABLED
		if ($this->cache_dir != '') {
			$cache_file = $this->cache_dir . '/rsscache_' . md5($rss_url);
			$timedif = @(time() - filemtime($cache_file));
			if ($timedif < $this->cache_time) {
				// cached file is fresh enough, return cached array
				$result = unserialize(join('', file($cache_file)));
				// set 'cached' to 1 only if cached file is correct
				if ($result) $result['cached'] = 1;
			} else {
				// cached file is too old, create new
				$result = $this->Parse($rss_url);
				$serialized = serialize($result);
				if ($f = @fopen($cache_file, 'w')) {
					fwrite ($f, $serialized, strlen($serialized));
					fclose($f);
				}
				if ($result) $result['cached'] = 0;
			}
		}
		// If CACHE DISABLED >> load and parse the file directly
		else {
			$result = $this->Parse($rss_url);
			if ($result) $result['cached'] = 0;
		}
		// return result
		return $result;
	}
	
	// -------------------------------------------------------------------
	// Modification of preg_match(), that returns field with index 1
	// from the Matches array
	// -------------------------------------------------------------------
	function my_preg_match ($pattern, $subject) {
		preg_match($pattern, $subject, $out);
		return $out[1];
	}

	// -------------------------------------------------------------------
	// Parse() is private method used by Get() to load and parse RSS file.
	// Don't use Parse() in your scripts - use Get($rss_file) instead.
	// -------------------------------------------------------------------
	function Parse ($rss_url) {

		// Open and load RSS file
		if ($f = @fopen($rss_url, 'r')) {
			$rss_content = '';
			while (!feof($f)) {
				$rss_content .= fgets($f, 4096);
			}
			fclose($f);

			// <CZ code page support: BEGIN>
				if ($this->cp != '') {
					$rss_content = AutoCzech($rss_content, $this->cp);
				}
			// </CZ code page support: END>

			// Parse CHANNEL info
			$result['title'] = $this->my_preg_match("'<channel>.+?<title>(.*?)</title>.+?</channel>'si", $rss_content);
			$result['link'] = $this->my_preg_match("'<channel>.+?<link>(.*?)</link>.+?</channel>'si", $rss_content);
			$result['description'] = $this->my_preg_match("'<channel>.+?<description>(.*?)</description>.+?</channel>'si", $rss_content);
			$result['language'] = $this->my_preg_match("'<channel>.+?<language>(.*?)</language>.+?</channel>'si", $rss_content);
			$result['encoding'] = $this->my_preg_match("'encoding=\"(.*?)\"'si", $rss_content);
			
			// Parse OPTIONAL info
			$result['copyright'] = $this->my_preg_match("'<channel>.+?<copyright>(.*?)</copyright>.+?</channel>'si", $rss_content);
			$result['managingEditor'] = $this->my_preg_match("'<channel>.+?<managingEditor>(.*?)</managingEditor>.+?</channel>'si", $rss_content);
			$result['webMaster'] = $this->my_preg_match("'<channel>.+?<webMaster>(.*?)</webMaster>.+?</channel>'si", $rss_content);
			$result['rating'] = $this->my_preg_match("'<channel>.+?<rating>(.*?)</rating>.+?</channel>'si", $rss_content);
			$result['pubDate'] = $this->my_preg_match("'<channel>.+?<pubDate>(.*?)</pubDate>.+?</channel>'si", $rss_content);
			$result['lastBuildDate'] = $this->my_preg_match("'<channel>.+?<lastBuildDate>(.*?)</lastBuildDate>.+?</channel>'si", $rss_content);
			$result['pubDate'] = $this->my_preg_match("'<channel>.+?<docs>(.*?)</docs>.+?</channel>'si", $rss_content);
			
			// Parse TEXTINPUT info
			preg_match("'<textinput>(.*?)</textinput>'si", $rss_content, $out_textinfo);
			if ($out_textinfo[1]) {
				$result['textinput_title'] = $this->my_preg_match("'<title>(.*?)</title>'si", $out_textinfo[1]);
				$result['textinput_description'] = $this->my_preg_match("'<description>(.*?)</description>'si", $out_textinfo[1]);
				$result['textinput_name'] = $this->my_preg_match("'<name>(.*?)</name>'si", $out_textinfo[1]);
				$result['textinput_link'] = $this->my_preg_match("'<link>(.*?)</link>'si", $out_textinfo[1]);
			}
			// Parse IMAGE info
			preg_match("'<image>(.*?)</image>'si", $rss_content, $out_imageinfo);
			if ($out_imageinfo[1]) {
				$result['image_title'] = $this->my_preg_match("'<title>(.*?)</title>'si", $out_imageinfo[1]);
				$result['image_url'] = $this->my_preg_match("'<url>(.*?)</url>'si", $out_imageinfo[1]);
				$result['image_link'] = $this->my_preg_match("'<link>(.*?)</link>'si", $out_imageinfo[1]);
				$result['image_width'] = $this->my_preg_match("'<width>(.*?)</width>'si", $out_imageinfo[1]);
				$result['image_height'] = $this->my_preg_match("'<height>(.*?)</height>'si", $out_imageinfo[1]);
			}
			// Parse ITEMS
			preg_match_all("'<item>(.*?)</item>'si", $rss_content, $items);
			$rss_items = $items[1];
			$result['items_count'] = count($items[1]);
			$i = 0;
			$result['items'] = array(); // create array even if there are no items
			foreach($rss_items as $rss_item) {
				$result['items'][$i]['title'] = $this->my_preg_match("'<title>(.*?)</title>'si", $rss_item);
				$result['items'][$i]['link'] = $this->my_preg_match("'<link>(.*?)</link>'si", $rss_item);
				$result['items'][$i]['description'] = $this->my_preg_match("'<description>(.*?)</description>'si", $rss_item);
				$i++;
			}
			
			return $result;
		}
		else // Error in opening return False
		{
			return False;
		}
	}
}

?>